###############################################################################
#                                                                             #
#                   Deleting all security groups for Office 365               #
#                                    V3.91                                    #
#                       (C)2006-2017 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($Group)
$S="ErrorsCount[$($Group)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Deleting all security groups for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2017 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
## Install MSOnline Cmdlets 
function Install-MSOnlineCmdlets()
{
$Res=$false
$DotNET35Path="HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v3.5"
$DotNet35Present=((Test-Path $DotNET35Path) -and ((Get-ItemProperty $DotNET35Path).Install -eq "1"))
if ($DotNet35Present)
   {
   # Trying to import module
   if (-not(Get-Module -Name "MSOnline")){Get-Module -ListAvailable | Where-Object {$_.name -eq "MSOnline"} | Import-Module}
   # Module is not present
   if (-not(Get-Module -Name "MSOnline"))
      {
      Write-Host "MSOnline Powershell module is not present !" -Foregroundcolor "Red"
      $wc=New-Object System.Net.WebClient
      # Sign-in Assistant
      $SigninAssistantPath="HKLM:\SOFTWARE\Microsoft\MSOIdentityCRL"
      $SigninAssistantPresent=((Test-Path $SigninAssistantPath) -and ((Get-ItemProperty $SigninAssistantPath).TargetDir -ne $null))
      $MyCulture="0x{0:X}" -f(Get-Culture).LCID
      if (!$SigninAssistantPresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_64.msi"}
            else
            {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_32.msi"}
         $Destination=$env:TEMP+"\_SigninAssistant_.msi"
         Write-Host "Trying to download ""Microsoft Online Sign-in Assistant"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Sign-in Assistant installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Sign-in Assistant, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            } 
         $FileExists=Test-Path $Destination
         If ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # MSOnline module
      $MSOnlinePath="HKLM:\SOFTWARE\Microsoft\MSOnlinePowerShell"
      $MSOnlinePresent=((Test-Path $MSOnlinePath) -and ((Get-ItemProperty $MSOnlinePath).InstallPath -ne $null))
      if (!$MSOnlinePresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            {
            # 64 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x64-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.1.166.0
               $Source="http://download.connect.microsoft.com/pr/AdministrationConfig_3.msi?t=3edbe1ec-9076-441a-ba1d-6a8ae147d3ea&e=1497200079&h=81bba22aebadb1efceb708c41c45fa85"
               }
            }
            else
            {
            # 32 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               $Source="http://go.microsoft.com/fwlink/?linkid=236298"
               $Source=$Source+"&clcid="+$MyCulture
               }
            }
         $Destination=$env:TEMP+"\_MSOnline_.msi"
         write-host "Trying to download ""Microsoft Online Powershell Module"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Powershell Module installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Powershell Module, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            }
         $FileExists=Test-Path $Destination
         if ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # Import MSOnline Module
      $Mod=Import-Module "MSOnline" -PassThru
      if ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="MSOnline Powershell module is still not present !"
         Write-Host $S -Foregroundcolor "Red"
         Log_Message "ERROR" $S
         Write-Host "Press any key ..."
         Read-Host
         }
         else
         {
         $S="MSOnline Powershell module is now present and loaded !"
         Write-Host $S -Foregroundcolor "Green"
         Log_Message "OK" $S
         $Res=$true
         }
      }
      else
      {
      Write-Host "MSOnline Powershell module is present" -Foregroundcolor "Green"
      $Res=$true
      }
  }
  else
  {
   $S="Please, install Framework .NET V3.5 first !"
   Write-Host $S -ForegroundColor "Red"
   Log_Message "ERROR" $S
   Write-Host "Press any key ..."
   Read-Host
  }
return $Res
}
###############################################################################
function connect_msol
{
Param([string] $MyName,$MyPassword)
$error.Clear()
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
Connect-MsolService -Credential $Cred -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
   $S="Connection was successful to Microsoft Online Services"
	 log_message "OK" $S
   } 
   else 
   {
   $global:total_errors_count++
   $S="Connection was unsuccessful to Microsoft Online Services || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function delete_security_group
{
Param($Group)
$error.Clear()
Remove-MsolGroup -ObjectId $Group.ObjectId -Force
if ([String]::IsNullorEmpty($error[0]))
	{
  $S="Successfully delete security group ""$($Group.DisplayName)"""
	write-host $S
  log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
  $S="Deleting security group ""$($Group.DisplayName)"". Error = $($error[0])"
  log_message "ERROR" $S
  }
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.91"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%OFFICE365_PASSWORD%}"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%PROXY_PASSWORD%}"))
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
if (Install-MSOnlineCmdlets)
   {
   connect_msol $Office365LoginName $Office365LoginPassword
   if ($global:total_errors_count -eq 0)
      {
      $Groups=Get-MsolGroup -GroupType "Security"
      if ($Groups.Count -gt 0)
      	 {
         Write-Host "Security groups are : " -foregroundcolor "white"
         ForEach ($Group in $Groups)
                 {
                 Write-host -nonewline """$($Group.DisplayName)"" "
                 }
         Write-Host        
         Write-Host -nonewline "Delete all these groups ? Enter YES to continue : " -foregroundcolor "magenta"
         $ret=Read-Host
         $ret=$ret.ToLower()
         if ($ret -eq "yes")
            {  
            ForEach ($Group in $Groups)
                 {
                 delete_security_group $Group
                 }
            }     
         }
         else
         {
         Write-Host -nonewline "There are no security groups ! Press any key to continue : " -foregroundcolor "magenta"
         Read-Host
         }
      }
         
   log_errors_count
   $global:total_errors_count=$global:total_errors_count+$global:errors_count
   Remove-Module MsOnline
   }
   else
   {
   $S="MSOnline Powershell module cannot be loaded !"
   Write-Host $S -Foregroundcolor "Red"
   Log_Message "ERROR" $S
   }
log_total_errors_count