###############################################################################
#                                                                             #
#                   Deleting license for users for Office 365                 #
#                                    V3.91                                    #
#                       (C)2006-2017 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Deleting license for users for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2017 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
function credential_msol
{
Param([string] $MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
 return $Cred
}
###############################################################################
## Install MSOnline Cmdlets 
function Install-MSOnlineCmdlets()
{
$Res=$false
$DotNET35Path="HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v3.5"
$DotNet35Present=((Test-Path $DotNET35Path) -and ((Get-ItemProperty $DotNET35Path).Install -eq "1"))
if ($DotNet35Present)
   {
   # Trying to import module
   if (-not(Get-Module -Name "MSOnline")){Get-Module -ListAvailable | Where-Object {$_.name -eq "MSOnline"} | Import-Module}
   # Module is not present
   if (-not(Get-Module -Name "MSOnline"))
      {
      Write-Host "MSOnline Powershell module is not present !" -Foregroundcolor "Red"
      $wc=New-Object System.Net.WebClient
      # Sign-in Assistant
      $SigninAssistantPath="HKLM:\SOFTWARE\Microsoft\MSOIdentityCRL"
      $SigninAssistantPresent=((Test-Path $SigninAssistantPath) -and ((Get-ItemProperty $SigninAssistantPath).TargetDir -ne $null))
      $MyCulture="0x{0:X}" -f(Get-Culture).LCID
      if (!$SigninAssistantPresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_64.msi"}
            else
            {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_32.msi"}
         $Destination=$env:TEMP+"\_SigninAssistant_.msi"
         Write-Host "Trying to download ""Microsoft Online Sign-in Assistant"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Sign-in Assistant installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Sign-in Assistant, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            } 
         $FileExists=Test-Path $Destination
         If ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # MSOnline module
      $MSOnlinePath="HKLM:\SOFTWARE\Microsoft\MSOnlinePowerShell"
      $MSOnlinePresent=((Test-Path $MSOnlinePath) -and ((Get-ItemProperty $MSOnlinePath).InstallPath -ne $null))
      if (!$MSOnlinePresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            {
            # 64 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x64-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.1.166.0
               $Source="http://download.connect.microsoft.com/pr/AdministrationConfig_3.msi?t=3edbe1ec-9076-441a-ba1d-6a8ae147d3ea&e=1497200079&h=81bba22aebadb1efceb708c41c45fa85"
               }
            }
            else
            {
            # 32 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               $Source="http://go.microsoft.com/fwlink/?linkid=236298"
               $Source=$Source+"&clcid="+$MyCulture
               }
            }
         $Destination=$env:TEMP+"\_MSOnline_.msi"
         write-host "Trying to download ""Microsoft Online Powershell Module"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Powershell Module installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Powershell Module, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            }
         $FileExists=Test-Path $Destination
         if ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # Import MSOnline Module
      $Mod=Import-Module "MSOnline" -PassThru
      if ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="MSOnline Powershell module is still not present !"
         Write-Host $S -Foregroundcolor "Red"
         Log_Message "ERROR" $S
         Write-Host "Press any key ..."
         Read-Host
         }
         else
         {
         $S="MSOnline Powershell module is now present and loaded !"
         Write-Host $S -Foregroundcolor "Green"
         Log_Message "OK" $S
         $Res=$true
         }
      }
      else
      {
      Write-Host "MSOnline Powershell module is present" -Foregroundcolor "Green"
      $Res=$true
      }
  }
  else
  {
   $S="Please, install Framework .NET V3.5 first !"
   Write-Host $S -ForegroundColor "Red"
   Log_Message "ERROR" $S
   Write-Host "Press any key ..."
   Read-Host
  }
return $Res
}
###############################################################################
function UserMSolExist
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function UserHasAtLeastOneLicense
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
return (![string]::IsNullOrEmpty($User.Licenses))
}
###############################################################################
function UserHasLicense($email,$LicensePack)
{
# Param([string] $email,$LicensePack)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
$Res=$false
If ($User.Licenses.count -gt 0)
   {
   ForEach ($License In $User.Licenses)
           {
           If ($License.AccountSku.SkuPartNumber.ToLower() -eq $LicensePack.ToLower())
              {
              $Res=$true;
              }
           }
   }
return $Res
}
###############################################################################
function UserLicenses
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
$Res=""
If ($User.Licenses.count -gt 0)
   {
   ForEach ($License In $User.Licenses)
           {
           $Res=$Res+" "+$License.AccountSkuId
           }
   }
return $Res.Trim()
}
###############################################################################
function LicensePackDomain
{
$sku=Get-MsolAccountSku
if ($sku.count -gt 0)
   {
   return $sku[0].AccountSkuId.Split(":")[0]
   }
   else
   {
   return ""
   }
}
###############################################################################
function connect_msol
{
Param($Cred)
Connect-MsolService -Credential $Cred	-ErrorAction SilentlyContinue
}
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function removelicensepack_user
{
Param($UserDisplayName,$UserEmail,$LicensePack)
$error.Clear()
Set-MSolUserLicense -UserPrincipalName $UserEmail -RemoveLicenses $LicensePack -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
   $S="Successfully remove license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"""
   write-host $S -foregroundcolor "white"
   log_message "OK" $S
   }
   else
   {
   $global:errors_count++
   $S="Removing license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"". Error = $($error[0])"
   log_message "ERROR" $S
   }
}
###############################################################################
function display_licenses
{
Param($UserDisplayName,$UserEmail)
$error.Clear()
$Licenses=(Get-MsolUser -UserPrincipalName $UserEmail -ErrorAction SilentlyContinue).Licenses
if ($Licenses.Count -gt 0)
   {
   $S="Found license packs :"
   Write-Host $S
   log_message "OK" $S
   ForEach ($License in $Licenses)
           {
           $S="License pack : "+$License.AccountSkuId
           Write-Host $S
           log_message "OK" $S
           }
   }
}
###############################################################################
function exec
{
Param($UserDisplayName,$UserEmail,$Prefix,$LicensePacks)
$UserDisplayName=$UserDisplayName.Trim()
log_message "OFFICE365" "$($UserDisplayName) ($($UserEmail))"
display_user	$UserDisplayName $UserEmail
if (UserMSolExist($UserEmail))
   {
   $LicensePackList=$LicensePacks.Split(",")
   ForEach ($LicensePack in $LicensePackList)
           {
           if (![String]::IsNullOrEmpty($LicensePack))
            	 {
              if (UserHasLicense $UserEmail $LicensePack)
                    {
                    $LicensePack=$Prefix+':'+$LicensePack
                    removelicensepack_user $UserDisplayName $UserEmail $LicensePack
                    }
                    else
                    {
                    $global:errors_count++
                    $S="MSol User ""$($UserEmail)"" has no license pack : "+$LicensePack
                    write-host $S -foregroundcolor "red"
                    log_message "ERROR" $S
                    display_licenses $UserDisplayName $UserEmail
                    }
               }
           }      
   }
   else
   {
   $global:errors_count++
   $S="MSol User ""$($UserEmail)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserEmail)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.91"
#
# Office 365 params
#
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%OFFICE365_PASSWORD%}"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%PROXY_PASSWORD%}"))
#
# License pack parameters
#
$UsageLocation="FR"
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
$Credential=credential_msol $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if (Install-MSOnlineCmdlets)
   {
   Connect-MSolService -Credential $Credential
   if ($global:total_errors_count -eq 0)
      {
      $Prefix=LicensePackDomain  
      $LicensePacks="%OFFICE365_LICENCE_PACK_TO_REMOVE%"
      ### BEGIN_REPETITION ###
      $global:errors_count=0
      $UserDisplayName="%USER_TITLE% %USER_FIRST_NAME% %USER_LAST_NAME%"
      $UserEmail="%USER_EMAIL%"
      exec $UserDisplayName $UserEmail $Prefix $LicensePacks
      ### END_REPETITION ###      
      }
   }
   else
   {
   $global:total_errors_count++	
   }	
 log_total_errors_count 
